#
# Copyright (c) 2025 CESNET z.s.p.o.
#
# This file is a part of oarepo-workflows (see http://github.com/oarepo/oarepo-workflows).
#
# oarepo-workflows is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.
#
"""OARepo workflows model package.

This module provides database models and system fields for implementing
workflow functionality in OARepo records, including workflow states,
transitions and metadata handling capabilities.
"""

from __future__ import annotations
