#
# Copyright (c) 2025 CESNET z.s.p.o.
#
# This file is a part of oarepo-workflows (see http://github.com/oarepo/oarepo-workflows).
#
# oarepo-workflows is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.
#

"""Service presets for records.

This package provides service-level presets for configuring
record functionality in OARepo workflows. It handles service
class generation and configuration for record operations.
"""

from __future__ import annotations
